#import <Cocoa/Cocoa.h>
#include "USBMicroX.h"

#define USBMLog USBMX_LOG

/*
 *	USBMicroDevice
 *	Note: the Objective-C bindings use the 'userData'
 *	attribute of the USBmX_DeviceRef -- unsetting this
 *	will result in very weird things.  Use the userData
 *	field in the USBMicroDevice class if you need such
 *	functionality.
 */
@interface USBMicroDevice : NSObject
{
	USBmX_DeviceRef dev;
	id displayName;
@public
	void *userData;
}

/* Low level wrappers */
- (USBmX_DeviceRef)device;
- (int)command:(u_char[8])cmd response:(u_char[8])res;
- (int)initPorts;
- (int)writeA:(u_char)value;
- (int)writeB:(u_char)value;
- (int)writeABit:(u_char)and_term or:(u_char)or_term;
- (int)writeBBit:(u_char)and_term or:(u_char)or_term;
- (int)readA:(u_char *)value_out;
- (int)readB:(u_char *)value_out;
- (int)setBit:(u_char)line;
- (int)resetBit:(u_char)line;
- (int)directionA:(u_char)dir0 with:(u_char)dir1;
- (int)directionB:(u_char)dir0 with:(u_char)dir1;
//- (int)strobeWrite:(u_char)data
//	portSelect:(u_char)port
//	strobeLine:(u_char)line
//	pulseLength:(u_char)length;

/* Higher level wrappers */
- (u_char)A;
- (u_char)B;
- (id)setA:(u_char)v;
- (id)setB:(u_char)v;
- (id)outputA;
- (id)outputB;
- (id)inputA;
- (id)inputB;

/* Utility methods */
- (int)dumpInfo;
- (const char *)serialNumber;
- (const char *)deviceName;
- (const char *)manufacturer;
- (id)displayName; //CHANGE!
- (void)setDisplayName:(id)name;

@end

@interface USBMicro : NSObject {}
+ (int)deviceCount;
+ (NSArray *)deviceArray;
+ (int)rescan;
+ (USBMicroDevice *)defaultDevice;
+ (USBMicroDevice *)deviceWithSerialNumber:(const char *)serial;
+ (USBmX_ContextRef)context;
+ (void)deviceAddedNotification:(id)target selector:(SEL)selector;
+ (void)deviceRemovedNotification:(id)target selector:(SEL)selector;
+ (void)deviceWasAdded;
+ (void)deviceWasRemoved;
@end
