#import "USBMicroX_ObjC.h"

static USBmX_ContextRef USBMicro_Internal_ctx = NULL;
static NSMutableArray *USBMicro_Internal_devices = nil;
static id added_target;
static id removed_target;
static SEL added_sel;
static SEL removed_sel;

static USBMicroDevice* DevFromDevRef(USBmX_DeviceRef d)
{
	if(d) return (USBMicroDevice *)d->userData;
	return NULL;
}

static void DeviceAdded(USBmX_DeviceRef dev, void *cookie)
{
	USBMLog("device added");
	USBMicroDevice *d = [[USBMicroDevice alloc] init];
	((struct{@defs(USBMicroDevice)}*)d)->dev = dev;
	dev->userData = (void *)d;
	[USBMicro_Internal_devices addObject:d];
	[d release];
	CFShow((CFTypeRef)USBMicro_Internal_devices);
	[USBMicro deviceWasAdded];
}

static void DeviceRemoved(USBmX_DeviceRef dev, void *cookie)
{
	USBMLog("device removed");
	USBMicroDevice *d = DevFromDevRef(dev);
	[USBMicro_Internal_devices removeObject:d];
	CFShow((CFTypeRef)USBMicro_Internal_devices);
	[USBMicro deviceWasRemoved];
}

@implementation USBMicro

+ (USBmX_ContextRef)context
{
	if(!USBMicro_Internal_devices)
	{
		USBMicro_Internal_devices = [[NSMutableArray alloc] init];
	}

	if(!USBMicro_Internal_ctx)
	{
		if(USBMicro_Internal_ctx = USBmX_Create())
		{
			USBmX_DeviceAddedHandler(
				USBMicro_Internal_ctx,
				DeviceAdded,
				NULL);
			USBmX_DeviceRemovedHandler(
				USBMicro_Internal_ctx,
				DeviceRemoved,
				NULL);
			USBmX_Begin(USBMicro_Internal_ctx);
		} else {
			USBMLog("There was an error connecting to the USBMicro devices!");
		}
	}
	return USBMicro_Internal_ctx;
}

+ (int)rescan
{
	return USBmX_ScanForDevices([USBMicro context]);
}

+ (int)deviceCount
{
	return USBmX_DeviceCount([USBMicro context]);
}

+ (NSArray *)deviceArray
{
	return (NSArray *)USBMicro_Internal_devices;
}

+ (USBMicroDevice *)defaultDevice
{
	return DevFromDevRef(USBmX_DefaultDevice([USBMicro context]));
}

+ (USBMicroDevice *)deviceWithSerialNumber:(const char *)serial
{
	return DevFromDevRef(USBmX_DeviceWithSerial([USBMicro context], serial));
}

+ (void)deviceAddedNotification:(id)target selector:(SEL)selector
{
	added_target = target;
	added_sel = selector;
}

+ (void)deviceRemovedNotification:(id)target selector:(SEL)selector
{
	removed_target = target;
	removed_sel = selector;
}

+ (void)deviceWasAdded
{
	if(added_target && added_sel)
		[added_target performSelector:added_sel];
}

+ (void)deviceWasRemoved
{
	if(removed_target && removed_sel)
		[removed_target performSelector:removed_sel];
}

@end

@implementation USBMicroDevice

- (USBmX_DeviceRef)device
{
	return dev;
}

- (int)dumpInfo
{
	return USBmX_DeviceInfo(dev);
}

- (const char *)serialNumber
{
	return dev->serialNumber;
}

- (const char *)deviceName
{
	return dev->deviceName;
}

- (const char *)manufacturer
{
	return dev->manufacturer;
}

- (id)displayName
{
	if(displayName) return displayName;
	return [NSString stringWithCString:[self serialNumber]];
}

- (void)setDisplayName:(id)name
{
	if(displayName) [displayName release];
	displayName = [name retain];
}

- (int)command:(u_char[8])cmd response:(u_char[8])res
{
	return USBmX_RawCommand(dev, cmd, res);
}

- (int)initPorts
{
	return USBmX_InitPorts(dev);
}

- (int)writeA:(u_char)value
{
	return USBmX_WriteA(dev, value);
}

- (int)writeB:(u_char)value
{
	return USBmX_WriteB(dev, value);
}

- (int)writeABit:(u_char)and_term or:(u_char)or_term
{
	return USBmX_WriteABit(dev, and_term, or_term);
}

- (int)writeBBit:(u_char)and_term or:(u_char)or_term
{
	return USBmX_WriteBBit(dev, and_term, or_term);
}

- (int)readA:(u_char *)value_out
{
	return USBmX_ReadA(dev, value_out);
}

- (int)readB:(u_char *)value_out
{
	return USBmX_ReadB(dev, value_out);
}

- (int)setBit:(u_char)line
{
	return USBmX_SetBit(dev, line);
}

- (int)resetBit:(u_char)line
{
	return USBmX_ResetBit(dev, line);
}

- (int)directionA:(u_char)dir0 with:(u_char)dir1
{
	return USBmX_DirectionA(dev, dir0, dir1);
}

- (int)directionB:(u_char)dir0 with:(u_char)dir1
{
	return USBmX_DirectionA(dev, dir0, dir1);
}

/**************************************
***************************************
**************************************/

- (u_char)A
{
	u_char v = 0;
	USBmX_ReadA(dev, &v);
	return v;
}

- (u_char)B
{
	u_char v = 0;
	USBmX_ReadB(dev, &v);
	return v;
}

- (id)setA:(u_char)v
{
	USBmX_WriteA(dev, v);
	return self;
}

- (id)setB:(u_char)v
{
	USBmX_WriteB(dev, v);
	return self;
}

- (id)outputA
{
	USBmX_DirectionA(dev, 0xFF, 0xFF);
	return self;
}

- (id)outputB
{
	USBmX_DirectionB(dev, 0xFF, 0xFF);
	return self;
}

- (id)inputA
{
	USBmX_DirectionA(dev, 0x00, 0x00);
	return self;
}

- (id)inputB
{
	USBmX_DirectionB(dev, 0x00, 0x00);
	return self;
}

@end
