#import "USBGUIController.h"
#import "USBMXMisc.h"

#import "CommandController.h"
#import "IOController.h"

typedef int File_Writer_t(void *, const char *, int);
NSTextView *stdoutConsole = nil;


static int StdoutWriter(void *inFD, const char *buffer, int size)
{
	NSString *tmp = [[NSString alloc] initWithCString:buffer length:size];
	USBMXAppendNSTextView(stdoutConsole, tmp);
	[tmp release];
	return size;
}

@implementation USBGUIController

- (USBMicroDevice *)selectedDevice
{
	int r = [deviceTable selectedRow];
	if(r == -1) return nil;
	return [[USBMicro deviceArray] objectAtIndex:r];
}

- (void)deviceAdded
{
	[deviceTable reloadData];
}

- (void)deviceRemoved
{
	[deviceTable reloadData];
}

- (void)awakeFromNib
{
	stdoutConsole = console;
    stdout->_write = &StdoutWriter;
    stderr->_write = &StdoutWriter;
	
	[consoleDrawer setContentSize:NSMakeSize(300.0,136.0)];
	
	[USBMicro context];
	
	[USBMicro deviceAddedNotification:self selector:@selector(deviceAdded)];
	[USBMicro deviceRemovedNotification:self selector:@selector(deviceRemoved)];
}

- (void)startLongWait
{
	[progress startAnimation:self];
}

- (void)endLongWait
{
	[progress stopAnimation:self];
}

- (IBAction)toggleConsole:(id)sender
{
	switch([sender state])
	{
		case NSOnState:
			[consoleDrawer openOnEdge:NSMinYEdge];
			break;
		case NSOffState:
			[consoleDrawer close];
			break;
	}
}

- (IBAction)clearConsole:(id)sender
{
	USBMXClearNSTextView(stdoutConsole);
}

- (IBAction)clearMoreInfoConsole:(id)sender
{
	USBMXClearNSTextView(moreInfoConsole);
}

- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{
	USBMicroDevice *d = [[USBMicro deviceArray] objectAtIndex:rowIndex];
	if(d) {
		return [d displayName];
	} return @"Invalid";
}

- (void)tableView:(NSTableView *)aTableView
    setObjectValue:anObject
    forTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{
	USBMicroDevice *d = [[USBMicro deviceArray] objectAtIndex:rowIndex];
	if(d) {
		[d setDisplayName:anObject];
	}
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[USBMicro deviceArray] count];
}

- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	return YES;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	USBMLog("Changing devices");
	USBMicroDevice *d = [self selectedDevice];
	if(!d) goto noSelection;
	[deviceField setStringValue:[NSString stringWithCString:[d deviceName]]];
	[manufacturerField setStringValue:[NSString stringWithCString:[d manufacturer]]];
	[serialField setStringValue:[NSString stringWithCString:[d serialNumber]]];
	[moreInfoButton setEnabled:YES];
	[mainTabView selectTabViewItemAtIndex:1];
	return;
noSelection:
	USBMLog("No device selected");
	[deviceField setStringValue:@""];
	[manufacturerField setStringValue:@""];
	[serialField setStringValue:@""];
	[moreInfoButton setEnabled:NO];
	[mainTabView selectTabViewItemAtIndex:0];
	[moreInfoPanel orderOut:nil];
	return;
}

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	int s = [[tabViewItem identifier] intValue];
	switch(s) {
		case 1: //command
			[commandController activate];
			break;
		case 2: //io
			[ioController activate];
			break;
		case 3: //lcd
		case 4: //stepper
		case 5: //spi
		default:
			break;
	}
}

- (IBAction)moreInfo:(id)sender
{
	USBMicroDevice *d = [self selectedDevice];
	if(!d) return;

	[self startLongWait];

	[moreInfoPanel setTitle:[NSString stringWithFormat:@"Info for %@", [d displayName]]];
	[self clearMoreInfoConsole:nil];

	NSTextView *oldConsole = stdoutConsole;
	stdoutConsole = moreInfoConsole;
	[d dumpInfo];
	[stdoutConsole scrollRangeToVisible:NSMakeRange(0, 1)];
	stdoutConsole = oldConsole;
	[moreInfoPanel makeKeyAndOrderFront:nil];	

	[self endLongWait];
}

- (IBAction)refreshList:(id)sender
{
	[USBMicro rescan];
}

@end
