Attribute VB_Name = "HIDInterface"
'================================================================================
'
' AccessIODevice
'
' This module declares the Open, Read, Write, and Close calls for the USB device
' These user calls are translated into OS system calls
'
'================================================================================



' Declare module-wide variables

Private HidHandle As Long

' USB Communication
Public OutBuffer(10) As Byte
Public InBuffer(10) As Byte
Public DeviceName$
Public DeviceManufact$
Public DeviceVID
Public DevicePID
Public DeviceDID
Public DeviceSerNum$
Public Attr As HidD_Attributes
Public DeviceFound As Boolean


'--------------------------------------------------------------------------------
'
' USB Transfer
'
'--------------------------------------------------------------------------------
Public Sub WriteReadUSB()

    If DeviceFound = True Then

        Call WriteUSBdevice(AddressFor(OutBuffer(0)), 8)
        Debug.Print "+"; _
                    TwoHexCharacters(OutBuffer(0)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(1)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(2)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(3)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(4)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(5)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(6)); _
                    " "; _
                    TwoHexCharacters(OutBuffer(7))
        DoEvents

        Call ReadUSBdevice(AddressFor(InBuffer(0)), 8)
        Debug.Print "-"; _
                    TwoHexCharacters(InBuffer(0)); _
                    " "; _
                    TwoHexCharacters(InBuffer(1)); _
                    " "; _
                    TwoHexCharacters(InBuffer(2)); _
                    " "; _
                    TwoHexCharacters(InBuffer(3)); _
                    " "; _
                    TwoHexCharacters(InBuffer(4)); _
                    " "; _
                    TwoHexCharacters(InBuffer(5)); _
                    " "; _
                    TwoHexCharacters(InBuffer(6)); _
                    " "; _
                    TwoHexCharacters(InBuffer(7))
        Debug.Print

        DoEvents

        If InBuffer(0) <> OutBuffer(0) Then
            Call WriteUSBdevice(AddressFor(OutBuffer(0)), 8)
            Debug.Print "!1 "; TwoHexCharacters(OutBuffer(0))
            Call ReadUSBdevice(AddressFor(InBuffer(0)), 8)
            Debug.Print "!1 "; TwoHexCharacters(InBuffer(0))
        End If

        If InBuffer(0) <> OutBuffer(0) Then
            Call WriteUSBdevice(AddressFor(OutBuffer(0)), 8)
            Debug.Print "!2 "; TwoHexCharacters(OutBuffer(0))
            Call ReadUSBdevice(AddressFor(InBuffer(0)), 8)
            Debug.Print "!2 "; TwoHexCharacters(InBuffer(0))
        End If

    End If

End Sub




'--------------------------------------------------------------------------------
'
' Sub: OpenUSBdevice
'
' This function searches the system HID tables for NameOfDevice$
' If found, then it opens the device and returns TRUE when the
' manufacturer, VID, PID, DID, and Serial # match, else it returns FALSE
'
'--------------------------------------------------------------------------------

Public Function OpenUSBdevice(NameOfDevice$, ManufactOfDevice$, _
                                VIDOfDevice, PIDOfDevice, DIDOfDevice, _
                                SerNumOfDevice$) As Boolean

    ' Locals
    Dim HidGuid As Guid
    Dim Success As Boolean
    Dim Opened As Boolean
    Dim Buffer(256) As Byte
    Dim DeviceInterfaceData As Device_Interface_Data
    Dim FunctionClassDeviceData As Device_Interface_Detail
    Dim SA As Security_Attributes


    ' First, get the HID class identifier
    Call HidD_GetHidGuid(HidGuid.Data(0))

    ' Get a handle for the Plug and Play node, request currently active HID devices
    PnPHandle& = SetupDiGetClassDevs(HidGuid.Data(0), 0, 0, &H12)

    If (PnPHandle& = -1) Then ErrorExit ("Could not attach to PnP node")


    HidEntry& = 0
    Opened = False

    ' Length of data structure in bytes.
    DeviceInterfaceData.cbsize = 28


    ' Look through the table of HID devices
    Do While SetupDiEnumDeviceInterfaces(PnPHandle&, 0, HidGuid.Data(0), HidEntry&, DeviceInterfaceData.cbsize)

        ' There is a device here, get it's name
        FunctionClassDeviceData.cbsize = 5

        Success = SetupDiGetDeviceInterfaceDetail(PnPHandle&, DeviceInterfaceData.cbsize, _
                FunctionClassDeviceData.cbsize, UBound(FunctionClassDeviceData.DataPath), BytesReturned&, 0)

        If (Success = 0) Then ErrorExit ("Could not get the name of this HID device")


        ' Convert returned C string to Visual Basic String
        hidname$ = ""
        i& = 0

        Do While FunctionClassDeviceData.DataPath(i&) <> 0
            hidname$ = hidname$ & Chr$(FunctionClassDeviceData.DataPath(i&))
            i& = i& + 1
        Loop


        ' Can now open this HID device
        HidHandle& = CreateFile(hidname$, &HC0000000, 3, SA, 3, 0, 0)


        If (HidHandle <> -1) Then
            'ErrorExit ("Could not open HID device")

            ' Get Product String
            If HidD_GetProductString(HidHandle&, AddressFor(Buffer(0)), UBound(Buffer)) Then
                DeviceName$ = ""
                i& = 0
                Do While Buffer(i&) <> 0
                    DeviceName$ = DeviceName$ & Chr$(Buffer(i&))
                    i& = i& + 2
                Loop
            End If

            ' Get Manuf String
            If HidD_GetManufacturerString(HidHandle&, AddressFor(Buffer(0)), UBound(Buffer)) Then
                DeviceManufact$ = ""
                i& = 0
                Do While Buffer(i&) <> 0
                    DeviceManufact$ = DeviceManufact$ & Chr$(Buffer(i&))
                    i& = i& + 2
                Loop
            End If


            ' Get VID, PID, DID
            If HidD_GetAttributes(HidHandle&, Attr) Then
                DeviceVID = Attr.VendorID(1)
                DeviceVID = (DeviceVID * 256 + Attr.VendorID(0))

                DevicePID = Attr.ProductID(1)
                DevicePID = (DevicePID * 256 + Attr.ProductID(0))

                DeviceDID = Attr.VersionNumber(1)
                DeviceDID = (DeviceDID * 256 + Attr.VersionNumber(0))
            End If

            ' Get Serial String
            If HidD_GetSerialNumberString(HidHandle&, AddressFor(Buffer(0)), UBound(Buffer)) Then
                DeviceSerNum$ = ""
                i& = 0
                Do While Buffer(i&) <> 0
                    DeviceSerNum$ = DeviceSerNum$ & Chr$(Buffer(i&))
                    i& = i& + 2
                Loop
            End If

            ' Don't allow all nulls
            If (NameOfDevice$ = "") And _
               (ManufactOfDevice$ = "") And _
               (VIDOfDevice = 0) And _
               (PIDOfDevice = 0) And _
               (DIDOfDevice = 0) And _
               (SerNumOfDevice$ = "") Then
               Opened = False
            Else
               Opened = True
            End If


            ' Open via Product String?
            If (NameOfDevice$ <> "") And _
               (StrComp(DeviceName$, NameOfDevice$) <> 0) Then
                    Opened = False
            End If

            ' Open via Manufacturer String?
            If (ManufactOfDevice$ <> "") And _
               (StrComp(DeviceManufact$, ManufactOfDevice$) <> 0) Then
                    Opened = False
            End If

            ' Open via VID?
            If (VIDOfDevice <> 0) And _
               (DeviceVID <> VIDOfDevice) Then
                    Opened = False
            End If

            ' Open via PID?
            If (PIDOfDevice <> 0) And _
               (DevicePID <> PIDOfDevice) Then
                    Opened = False
            End If

            ' Open via DID?
            If (DIDOfDevice <> 0) And _
               (DeviceDID <> DIDOfDevice) Then
                    Opened = False
            End If

            If (Opened = False) Then Call CloseHandle(HidHandle&) ' Was not OUR HID device

        End If

        HidEntry& = HidEntry& + 1 ' Check next entry

        ' This was our device, stop searching
        If (Opened = True) Then Exit Do


    Loop 'SetupDiEnumDeviceInterfaces returns FALSE when there are no more entries


    SetupDiDestroyDeviceInfoList (PnPHandle&)

    OpenUSBdevice = Opened


End Function



'--------------------------------------------------------------------------------
'
' Sub: ReadUSBdevice

' This subroutine reads from an opened USB device.
' This routine gets an Input Report from the USB device and returns the data.
' NOTE that ReadFile is a BLOCKING system call, ie it will wait for the USB device to respond.
' Do not configure the USB device to "Generate report only on change" since the program
' will appear to 'hang'.
' Use a local buffer so that the ReportID (=0) at ReportBuffer(0) may be removed.
'
'--------------------------------------------------------------------------------

Public Sub ReadUSBdevice(BufferPtr&, ByteCount&)

    ' Locals
    Dim ReportBuffer(256) As Byte

    ' Too many bytes
    If ByteCount& > 254 Then
        ErrorExit ("Maximum ByteCount for ReadUSBdevice is 254")
    End If

    ' Too few bytes
    If ByteCount& < 1 Then
        ErrorExit ("Minimum ByteCount for ReadUSBdevice is 1")
    End If

    ' Read
    Success = ReadFile(HidHandle&, AddressFor(ReportBuffer(0)), ByteCount& + 1, BytesReturned&, 0)

    ' Test for ReadFile failure
    If (Success = 0) Then
        ErrorExit ("Could not get an Input Report")
    End If

    ' Copy buffer
    Call CopyBuffer(AddressFor(ReportBuffer(1)), BufferPtr&, BytesReturned& - 1)

End Sub



'--------------------------------------------------------------------------------
'
' Sub: WriteUSBdevice

' This subroutine writes to an opened USB device
'
' Copy the user buffer into a local buffer so that a ReportID (=0)
' may be prepended.
' The first byte will contain the ReportID (=0).
'
'--------------------------------------------------------------------------------

Public Sub WriteUSBdevice(BufferPtr&, ByteCount&)

    ' Locals
    Dim ReportBuffer(256) As Byte

    ' Too many bytes
    If ByteCount& > 254 Then
        ErrorExit ("Maximum ByteCount for WriteUSBdevice is 254")
    End If

    ' Copy buffer
    Call CopyBuffer(BufferPtr&, AddressFor(ReportBuffer(1)), ByteCount&)

    ' Add report ID
    ReportBuffer(0) = 0

    ' Write
    Success = WriteFile(HidHandle&, AddressFor(ReportBuffer(0)), ByteCount& + 1, BytesWritten&, 0)

    ' Test for WriteFile failure
    If (Success = 0) Then
        ErrorExit ("Could not write an Output Report")
    End If

End Sub



'--------------------------------------------------------------------------------
'
' Close USB device
'
' This subroutine closes the USB device
'
'--------------------------------------------------------------------------------

Public Sub CloseUSBdevice()

    Call CloseHandle(HidHandle&)
    HidHandle = 0

End Sub
